//------------------------------------------------------------------------------
//  :	Syn/main.c 
// :	- 
//   :
//    1.     ("" +/-)
//    2.     EEPROM    ("")
//    3.     TIMER1 (  A)  
//        ,  .
//
//  :
//        : 30 - 150     62.5 
//                     EEPROM 
//                     
//         " +/-"  0.5   
//                   1 .
//
//    3.7.5 -    
//------------------------------------------------------------------------------
// :	ATtiny2313
//  : 16
// :		3.7.6
//
// :		 .., ku_@mail.ru
//
//  .., 2011
//------------------------------------------------------------------------------

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/eeprom.h>

#define SaveBut     PD2

//--------------------------------------------------------------------------------
//   (  )  , .. 
// (   !)      
#define NoClick           0

#define FPlusShortClick   0b00001000
#define FMinusShortClick  0b00010000

#define FPlusLongClick    0b00000001
#define FMinusLongClick   0b00000010

#define FPButsMask        0b00011000
//--------------------------------------------------------------------------------
#define PAUSE 10          //        /

#define NMin 479          //        
#define NMax 2399         //   

unsigned int N;         //  
unsigned int EEMEM Neep = 1599;
unsigned char M = 249; // 124; //  
unsigned char d;
unsigned char rStep = 0; //   N    
unsigned char FChangeFlag = 1, PChangeFlag = 1;
unsigned char P;        //    (%)
unsigned long F;        //    ()

//-----------------------------------------------------------------------------
//   ""  ""
//-----------------------------------------------------------------------------
unsigned char get_key(void){
  unsigned char i, bFlag = PIND & FPButsMask;
  if(bFlag == FPButsMask){ //      
    rStep = 0; return NoClick;
  }
  _delay_ms(PAUSE); //   
  if((PIND & FPButsMask) == FPButsMask){ // - ...
    rStep = 0; return NoClick;
  }
  if(rStep){  //  " "
    if(rStep < 16)  //  
      rStep++;
    _delay_ms(200); // ,     
    return (bFlag >> 3);  //   
  }
  for(i = 0; i < 50; i++){   //      0.5 
    if((PIND & FPButsMask) == FPButsMask) // -   
      return bFlag;
    _delay_ms(10);
  }
  rStep = 2; //  " "
  return (bFlag >> 3);    
}
//-----------------------------------------------------------------------------
//   ""
//-----------------------------------------------------------------------------
void save(void){
  if(PIND & _BV(SaveBut)) //      
    return;
  _delay_ms(PAUSE); //   
  if(PIND & _BV(SaveBut))  // - ...
    return;
  eeprom_write_word(&Neep, N);    //  N  EEPROM
}
//------------------------------------------------------------------------------
//     TIMER1   A
//         OCR1A   
//    
//------------------------------------------------------------------------------
ISR(TIMER1_COMPA_vect){
  TIMSK &= ~_BV(OCIE1A);           //      OCR1A	      
  OCR1AH = (N >> 8) & 0x00FF;      //     
  OCR1AL = N & 0x00FF;
  FChangeFlag = 0;
}
//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
int main (void) {
	DDRB = 0b00001100; PORTB = 0b11110011;  //   B
  DDRD = 0b00000000; PORTD = 0b11111111;  //   D -  .   

  //    - 1
  TCCR1A = _BV(COM1A0);           // COM1A1 = 0  COM1A0 = 1 -  OC1A   
  TCCR1B = _BV(WGM12) |           // WGM = 0100 -  CTC
           _BV(CS12)|             // CS = 111 -    ( )
           _BV(CS11)|
           _BV(CS10);
  N = eeprom_read_word(&Neep);    //   N  EEPROM 
//  OCR1AH = (N >> 8) & 0x00FF;     //  
//  OCR1AL = N & 0x00FF;

  //    - 0
  TCCR0A = _BV(COM0A0) |          // COM0A1 = 0  COM0A0 = 1 -  OC0A   
           _BV(WGM01);            // WGM = 010 -  CTC
  TCCR0B = _BV(CS02)|             // CS = 101 -   
           _BV(CS00);             //   1024
  OCR0A = M;                      //  
  
  FChangeFlag = 1;

  sei();
	while(1) {
    switch(get_key()){  //   
      case FPlusShortClick: //     
        if(N < NMax){
          N++; FChangeFlag = 1;
        }
        break;
      case FMinusShortClick: //     
        if(N > NMin){
          N--; FChangeFlag = 1;
        }
        break;
      case FPlusLongClick: //     
        if(N <= (NMax - rStep)){
          N += rStep; FChangeFlag = 1;
        }else{
          N = NMax; FChangeFlag = 1;
        }
        break;
      case FMinusLongClick: //     
        if(N >= (NMin + rStep)){
          N -= rStep; FChangeFlag = 1;
        }else{
          N = NMin; FChangeFlag = 1;
        }
        break;
    }
    if(FChangeFlag){
      TIMSK |= _BV(OCIE1A);   //       OCR1A
    }
    save(); //    "Save"    ,  
	}
	return 0;
}
